angular.module('ftsmon', ['ngRoute', 'ftsmon.resources', 'ftsmon.plots', 'ftsmon.global_filter', 'ui.bootstrap']).
config(function($routeProvider) {
    $routeProvider.
        when('/',                     {templateUrl: STATIC_ROOT + 'html/overview.html',
                                       controller:  OverviewCtrl,
                                       resolve:     OverviewCtrl.resolve}).
        when('/jobs',                 {templateUrl: STATIC_ROOT + 'html/jobs/index.html',
                                       controller:  JobListCtrl,
                                           resolve:     JobListCtrl.resolve}).
        when('/job/:jobId',           {templateUrl: STATIC_ROOT + 'html/jobs/view.html',
                                       controller:  JobViewCtrl,
                                       resolve:     JobViewCtrl.resolve}).

        when('/transfers',            {templateUrl: STATIC_ROOT + 'html/transfers.html',
                                       controller:  TransfersCtrl,
                                       resolve:     TransfersCtrl.resolve}).

        when('/optimizer/',           {templateUrl: STATIC_ROOT + 'html/optimizer/optimizer.html',
                                       controller:  OptimizerCtrl,
                                       resolve:     OptimizerCtrl.resolve}).
        when('/optimizer/detailed',   {templateUrl: STATIC_ROOT + 'html/optimizer/detailed.html',
                                       controller:  OptimizerDetailedCtrl,
                                       resolve:     OptimizerDetailedCtrl.resolve}).
        when('/optimizer/streams',    {templateUrl: STATIC_ROOT + 'html/optimizer/streams.html',
                                       controller:  OptimizerStreamsCtrl,
                                       resolve:     OptimizerStreamsCtrl.resolve}).

        when('/errors/',              {redirectTo:  ErrorsCtrl.redirectTo}).
        when('/errors/pairs',         {templateUrl: STATIC_ROOT + 'html/errors/pairs.html',
                                       controller:  ErrorsCtrl,
                                       resolve:     ErrorsCtrl.resolve}).
        when('/errors/list',          {templateUrl: STATIC_ROOT + 'html/errors/list.html',
                                       controller:  ErrorsForPairCtrl,
                                       resolve:     ErrorsForPairCtrl.resolve}).

        when('/config/audit',         {templateUrl: STATIC_ROOT + 'html/config/audit.html',
                                       controller:  ConfigAuditCtrl,
                                       resolve:     ConfigAuditCtrl.resolve}).
        when('/config/status',        {templateUrl: STATIC_ROOT + 'html/config/status.html',
                                       controller:  ConfigStatusCtrl,
                                       resolve:     ConfigStatusCtrl.resolve}).
        when('/config/links',         {templateUrl: STATIC_ROOT + 'html/config/links.html',
                                       controller:  ConfigLinksCtrl,
                                       resolve:     ConfigLinksCtrl.resolve}).
        when('/config/limits',        {templateUrl: STATIC_ROOT + 'html/config/limits.html',
                                       controller:  ConfigLimitsCtrl,
                                       resolve:     ConfigLimitsCtrl.resolve}).
        when('/config/range',         {templateUrl: STATIC_ROOT + 'html/config/range.html',
                                       controller:  ConfigRangeCtrl,
                                       resolve:     ConfigRangeCtrl.resolve}).
        when('/config/gfal2',         {templateUrl: STATIC_ROOT + 'html/config/gfal2.html',
                                       controller:  Gfal2Ctrl,
                                       resolve:     Gfal2Ctrl.resolve}).
        when('/config/activities',    {templateUrl: STATIC_ROOT + 'html/config/activities.html',
                                       controller:  ActivitiesCtrl,
                                       resolve:     ActivitiesCtrl.resolve}).
        when('/config/activities/:vo',{templateUrl: STATIC_ROOT + 'html/config/vo_activities.html',
                                       controller:  VoActivePerActivitiesCtrl,
                                       resolve:     VoActivePerActivitiesCtrl.resolve}).

        when('/statistics/overview',  {templateUrl: STATIC_ROOT + 'html/statistics/overview.html',
                                       controller:  StatsOverviewCtrl,
                                       resolve:     StatsOverviewCtrl.resolve}).
        when('/statistics/servers',   {templateUrl: STATIC_ROOT + 'html/statistics/servers.html',
                                       controller:  StatsServersCtrl,
                                       resolve:     StatsServersCtrl.resolve}).
        when('/statistics/database',  {templateUrl: STATIC_ROOT + 'html/statistics/database.html',
                                       controller:  StatsDatabaseCtrl,
                                       resolve:     StatsDatabaseCtrl.resolve}).
        when('/statistics/vos',       {templateUrl: STATIC_ROOT + 'html/statistics/vos.html',
                                       controller:  StatsVosCtrl,
                                       resolve:     StatsVosCtrl.resolve}).
       when('/statistics/volume',     {templateUrl: STATIC_ROOT + 'html/statistics/volume.html',
                                       controller:  TransferVolumeCtrl,
                                       resolve:     TransferVolumeCtrl.resolve}).

        when('/overview/atlas',       {templateUrl: STATIC_ROOT + 'html/overview/atlas.html',
                                       controller:  OverviewAtlasCtrl,
                                       resolve:     OverviewAtlasCtrl.resolve}).

        when('/500',                    {templateUrl: STATIC_ROOT + 'html/500.html'}).

        otherwise({templateUrl: STATIC_ROOT + 'html/404.html'});
})
.filter('escape', function() {
    return function(v) {
        if (typeof v == 'undefined' || v === null) {
            return "";
        }
        return window.encodeURIComponent(v);
    }
})
.filter('hex', function() {
	return function(value) {
		hex = value.toString(16);
		return ("0000".substr(0, 4 - hex.length) + hex).toUpperCase();
	};
})
.filter('slice', function() {
    return function(arr, start, end) {
        if (typeof(end) == 'undefined')
            end = 999;

        var sliced = {};
        var i = 0;
        for (var k in arr) {
            if (i >= start && i <= end)
                sliced[k] = arr[k];
            i++;
        }
        return sliced;
    }
})
.filter('objlength', function() {
    return function(obj) {
        var i = 0;
        for (var k in obj)
            i++;
        return i;
    }
})
.directive('optionalNumber', function() {
    return {
        restrict: 'A',
        replace: false,
        template: '{{ value }} {{ suffix }}',
        scope: {},
        link: function(scope, element, attr) {
            scope.decimals = scope.$eval(attr.decimals);
            attr.$observe('optionalNumber', function(val) {
                scope.value = val;
                if (scope.value == null || scope.value == undefined || scope.value == '') {
                    scope.value = '-';
                }
                else if (scope.value == 0 && attr.displayZero != 'true') {
                    scope.value = '-';
                }
                else {
                    scope.value = Number(scope.value).toFixed(scope.decimals);
                    scope.suffix = attr.suffix;
                }
            });
        }
    }
})
.directive('orderBy', function($location) {
    return {
        restrict: 'A',
        scope: {},
        replace: true,
        link: function(scope, element, attr) {
            var content = element.text();
            var field = attr.orderBy;
            var title = attr.title;
            var orderedBy = $location.$$search.orderby;

            if (!orderedBy)
                orderedBy = '';

            if (!title)
                title = 'Order by ' + content;

            // Differentiate between field and ordering (asc/desc)
            var orderedDesc = false;
            if (orderedBy && orderedBy[0] == '-') {
                orderedDesc = true;
                orderedBy = orderedBy.slice(1);
            }

            // Now, add the icon if this is the field used for the ordering
            var icon = '';
            if (orderedBy == field) {
                if (orderedDesc)
                    icon = '<i class="icon-arrow-down"></i>';
                else
                    icon = '<i class="icon-arrow-up"></i>';
            }

            // HTML
            var html = '<span class="orderby" title="' + title + '">' + icon + content + '</span>';
            var replacement = angular.element(html);

            // Bind the method
            replacement.bind('click', function() {
                if (orderedDesc && field == orderedBy)
                    $location.search('orderby', field);
                else
                    $location.search('orderby', '-' + field);
                scope.$apply();
            });

            // Replace
            element.replaceWith(replacement);
        }
    }
})
.run(function($rootScope, $location) {
    $rootScope.searchJob = function() {
        $location.path('/job/' + $rootScope.jobId).search({});
    }
})
.filter('safeFilter', function($filter) {
    return function(list, expr) {
        if (typeof(list) == 'undefined')
            return [];
        return $filter('filter')(list, expr);
    }
})
.filter('filesize', function($filter) {
    return function(bytes) {
    	if (bytes == null)
    		return '-';
    	else if (bytes < 1024)
            return bytes.toString() + ' bytes';
        else if (bytes < 1048576)
            return (bytes / 1024.0).toFixed(2).toString() + ' KiB';
        else if (bytes < 1073741824)
            return (bytes / 1048576.0).toFixed(2).toString() + ' MiB';
        else
            return (bytes / 1073741824.0).toFixed(2).toString() + ' GiB';
    }
})
;

/** Refresh interval in ms */
var REFRESH_INTERVAL = 60000;

/** First letter uppercase */
function firstUpper(str)
{
    return str.charAt(0).toUpperCase() + str.substr(1).toLowerCase();
}

/** Show loading **/
var nLoading = 0;
function loading($rootScope)
{
    nLoading += 1;
    if (nLoading)
        $rootScope.loading = true;
}

function stopLoading($rootScope, headers)
{
    if (nLoading > 0)
        nLoading -= 1;
    if (!nLoading) {
        $rootScope.loading = false;
        var now = new Date();
        $rootScope.lastRefresh = "Generated at " + now.toLocaleTimeString();
        if (typeof(headers) != 'undefined' && 'x-host' in headers)
            $rootScope.lastRefresh += " (" + headers['x-host'] + ")";
    }
}

/** Join states with commas */
function joinStates(states)
{
    var str = '';
    if (states.submitted)
        str += 'SUBMITTED,';
    if (states.ready)
        str += 'READY,';
    if (states.staging)
        str += 'STAGING,';
    if (states.active)
        str += 'ACTIVE,';
    if (states.canceled)
        str += 'CANCELED,';
    if (states.failed)
        str += 'FAILED,';
    if (states.finished)
        str += 'FINISHED,';
    if (states.finisheddirty)
        str += 'FINISHEDDIRTY,';
    if (states.not_used)
        str += 'NOT_USED,';
    if (states.started)
        str += 'STARTED,';
    if (states.delete)
        str += 'DELETE,';
    if (str.length > 0)
        str = str.slice(0, -1);
    return str;
}

function statesFromString(str)
{
    var st = {ready: false, active: false, canceled: false, failed: false, finished: false, finisheddirty: false};

    if (typeof(str) != "undefined") {
        var states = str.split(',');

        for (var i in states) {
            if (states[i] == 'SUBMITTED')
                st.submitted= true;
            if (states[i] == 'READY')
                st.ready = true;
            if (states[i] == 'STAGING')
                st.staging = true;
            if (states[i] == 'ACTIVE')
                st.active = true;
            if (states[i] == 'CANCELED')
                st.canceled = true;
            if (states[i] == 'FAILED')
                st.failed = true;
            if (states[i] == 'FINISHED')
                st.finished = true;
            if (states[i] == 'FINISHEDDIRTY')
                st.finisheddirty = true;
            if (states[i] == 'NOT_USED')
                st.not_used = true;
            if (states[i] == 'STARTED')
                st.started = true;
            if (states[i] == 'DELETE')
                st.delete = true;
        }
    }

    return st;
}

/** Get default if undefined **/
function validString(v)
{
    if (typeof(v) == 'string')
        return v;
    else
        return '';
}

function undefinedAsZero(v)
{
    if (typeof(v) != 'undefined')
        return v;
    else
        return 0;
}

function withDefault(v, def)
{
    if (typeof(v) != 'undefined')
        return v;
    else
        return def;
}

/** Generic callbacks */
function genericSuccessMethod(deferred, $rootScope) {
    return function(data, headers) {
        deferred.resolve(data);
        if (typeof(headers) != 'undefined')
            stopLoading($rootScope, headers());
        else
            stopLoading($rootScope);
    }
}

function genericFailureMethod(deferred, $rootScope, $location) {
    return function (response) {
        if (deferred)
            deferred.resolve(false);
        stopLoading($rootScope);
        if (response.status == 404)
            $location.path('/404');
        else if (response.status == 500)
            $location.path('/500');
        else
            $location.path('/generic-error');
    }
}
